@@ -67,10 +67,8 @@ public:
   char               *conn_keys_first_ptr;
   char               **conn_keys;
   SPIDER_CONN        **conns;
-  /* array of indexes of active servers */
+  /* for active-standby mode */
   uint               *conn_link_idx;
-  /* A bitmap indicating whether each active server have some higher
-  numbered server in the same "group" left to try (can fail over) */
   uchar              *conn_can_fo;
   void               **quick_targets;
   int                *need_mons;
@@ -84,7 +82,6 @@ public:
 #ifdef WITH_PARTITION_STORAGE_ENGINE
   SPIDER_PARTITION_HANDLER *partition_handler;
 #endif
-  /* Whether this ha_spider is the owner of its wide_handler. */
   bool                wide_handler_owner = FALSE;
   SPIDER_WIDE_HANDLER *wide_handler = NULL;
 
@@ -131,7 +128,11 @@ public:
   bool               have_second_range;
   KEY_MULTI_RANGE    mrr_second_range;
   spider_string      *mrr_key_buff;
+#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100000
   range_id_t         *multi_range_keys;
+#else
+  char               **multi_range_keys;
+#endif
 #else
   KEY_MULTI_RANGE    *multi_range_ranges;
 #endif
@@ -153,7 +154,7 @@ public:
 #ifdef HANDLER_HAS_CAN_USE_FOR_AUTO_INC_INIT
   bool               auto_inc_temporary;
 #endif
-  int                bulk_size= 0;
+  int                bulk_size;
   int                direct_dup_insert;
   int                store_error_num;
   uint               dup_key_idx;
@@ -167,13 +168,7 @@ public:
   ulonglong          *db_request_id;
   uchar              *db_request_phase;
   uchar              *m_handler_opened;
-  /* ids for use in HANDLER command */
   uint               *m_handler_id;
-  /*
-    aliases for use in HANDLER command, in the format of t%5u on
-    m_handler_id. So for example, if m_handler_id is 3, then the
-    corresponding m_handler_cid is t00003
-  */
   char               **m_handler_cid;
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
   bool               do_direct_update;
@@ -216,14 +211,14 @@ public:
   handler *clone(
     const char *name,
     MEM_ROOT *mem_root
-  ) override;
+  );
   const char **bas_ext() const;
   int open(
     const char* name,
     int mode,
     uint test_if_locked
-  ) override;
-  int close() override;
+  );
+  int close();
   int check_access_kind_for_connection(
     THD *thd,
     bool write_request
@@ -241,30 +236,30 @@ public:
     THD *thd,
     THR_LOCK_DATA **to,
     enum thr_lock_type lock_type
-  ) override;
+  );
   int external_lock(
     THD *thd,
     int lock_type
-  ) override;
+  );
   int start_stmt(
     THD *thd,
     thr_lock_type lock_type
-  ) override;
-  int reset() override;
+  );
+  int reset();
   int extra(
     enum ha_extra_function operation
-  ) override;
+  );
   int index_init(
     uint idx,
     bool sorted
-  ) override;
+  );
 #ifdef HA_CAN_BULK_ACCESS
   int pre_index_init(
     uint idx,
     bool sorted
   );
 #endif
-  int index_end() override;
+  int index_end();
 #ifdef HA_CAN_BULK_ACCESS
   int pre_index_end();
 #endif
@@ -273,39 +268,40 @@ public:
     const uchar *key,
     key_part_map keypart_map,
     enum ha_rkey_function find_flag
-  ) override;
+  );
   int index_read_last_map(
     uchar *buf,
     const uchar *key,
     key_part_map keypart_map
-  ) override;
+  );
   int index_next(
     uchar *buf
-  ) override;
+  );
   int index_prev(
     uchar *buf
-  ) override;
+  );
   int index_first(
     uchar *buf
-  ) override;
+  );
   int index_last(
     uchar *buf
-  ) override;
+  );
   int index_next_same(
     uchar *buf,
     const uchar *key,
     uint keylen
-  ) override;
+  );
   int read_range_first(
     const key_range *start_key,
     const key_range *end_key,
     bool eq_range,
     bool sorted
-  ) override;
-  int read_range_next() override;
+  );
+  int read_range_next();
   void reset_no_where_cond();
   bool check_no_where_cond();
 #ifdef HA_MRR_USE_DEFAULT_IMPL
+#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100000
   ha_rows multi_range_read_info_const(
     uint keyno,
     RANGE_SEQ_IF *seq,
@@ -314,7 +310,7 @@ public:
     uint *bufsz,
     uint *flags,
     Cost_estimate *cost
-  ) override;
+  );
   ha_rows multi_range_read_info(
     uint keyno,
     uint n_ranges,
@@ -323,23 +319,55 @@ public:
     uint *bufsz,
     uint *flags,
     Cost_estimate *cost
-  ) override;
+  );
+#else
+  ha_rows multi_range_read_info_const(
+    uint keyno,
+    RANGE_SEQ_IF *seq,
+    void *seq_init_param,
+    uint n_ranges,
+    uint *bufsz,
+    uint *flags,
+    COST_VECT *cost
+  );
+  ha_rows multi_range_read_info(
+    uint keyno,
+    uint n_ranges,
+    uint keys,
+    uint key_parts,
+    uint *bufsz,
+    uint *flags,
+    COST_VECT *cost
+  );
+#endif
   int multi_range_read_init(
     RANGE_SEQ_IF *seq,
     void *seq_init_param,
     uint n_ranges,
     uint mode,
     HANDLER_BUFFER *buf
-  ) override;
+  );
+#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100000
   int multi_range_read_next(
     range_id_t *range_info
-  ) override;
+  );
   int multi_range_read_next_first(
     range_id_t *range_info
   );
   int multi_range_read_next_next(
     range_id_t *range_info
   );
+#else
+  int multi_range_read_next(
+    char **range_info
+  );
+  int multi_range_read_next_first(
+    char **range_info
+  );
+  int multi_range_read_next_next(
+    char **range_info
+  );
+#endif
 #else
   int read_multi_range_first(
     KEY_MULTI_RANGE **found_range_p,
@@ -354,57 +382,57 @@ public:
 #endif
   int rnd_init(
     bool scan
-  ) override;
+  );
 #ifdef HA_CAN_BULK_ACCESS
   int pre_rnd_init(
     bool scan
   );
 #endif
-  int rnd_end() override;
+  int rnd_end();
 #ifdef HA_CAN_BULK_ACCESS
   int pre_rnd_end();
 #endif
   int rnd_next(
     uchar *buf
-  ) override;
+  );
   void position(
     const uchar *record
-  ) override;
+  );
   int rnd_pos(
     uchar *buf,
     uchar *pos
-  ) override;
+  );
   int cmp_ref(
     const uchar *ref1,
     const uchar *ref2
-  ) override;
-  int ft_init() override;
-  void ft_end() override;
+  );
+  int ft_init();
+  void ft_end();
   FT_INFO *ft_init_ext(
     uint flags,
     uint inx,
     String *key
-  ) override;
+  );
   int ft_read(
     uchar *buf
-  ) override;
+  );
   int pre_index_read_map(
     const uchar *key,
     key_part_map keypart_map,
     enum ha_rkey_function find_flag,
     bool use_parallel
-  ) override;
-  int pre_index_first(bool use_parallel) override;
-  int pre_index_last(bool use_parallel) override;
+  );
+  int pre_index_first(bool use_parallel);
+  int pre_index_last(bool use_parallel);
   int pre_index_read_last_map(
     const uchar *key,
     key_part_map keypart_map,
     bool use_parallel
-  ) override;
+  );
 #ifdef HA_MRR_USE_DEFAULT_IMPL
   int pre_multi_range_read_next(
     bool use_parallel
-  ) override;
+  );
 #else
   int pre_read_multi_range_first(
     KEY_MULTI_RANGE **found_range_p,
@@ -421,47 +449,47 @@ public:
     bool eq_range,
     bool sorted,
     bool use_parallel
-  ) override;
-  int pre_ft_read(bool use_parallel) override;
-  int pre_rnd_next(bool use_parallel) override;
+  );
+  int pre_ft_read(bool use_parallel);
+  int pre_rnd_next(bool use_parallel);
   int info(
     uint flag
-  ) override;
+  );
   ha_rows records_in_range(
     uint inx,
     const key_range *start_key,
     const key_range *end_key,
     page_range *pages
-  ) override;
+  );
   int check_crd();
-  int pre_records() override;
-  ha_rows records() override;
+  int pre_records();
+  ha_rows records();
 #ifdef HA_HAS_CHECKSUM_EXTENDED
-  int pre_calculate_checksum() override;
-  int calculate_checksum() override;
+  int pre_calculate_checksum();
+  int calculate_checksum();
 #endif
-  const char *table_type() const override;
-  ulonglong table_flags() const override;
+  const char *table_type() const;
+  ulonglong table_flags() const;
   ulong table_flags_for_partition();
   const char *index_type(
     uint key_number
-  ) override;
+  );
   ulong index_flags(
     uint idx,
     uint part,
     bool all_parts
-  ) const override;
-  uint max_supported_record_length() const override;
-  uint max_supported_keys() const override;
-  uint max_supported_key_parts() const override;
-  uint max_supported_key_length() const override;
-  uint max_supported_key_part_length() const override;
-  uint8 table_cache_type() override;
+  ) const;
+  uint max_supported_record_length() const;
+  uint max_supported_keys() const;
+  uint max_supported_key_parts() const;
+  uint max_supported_key_length() const;
+  uint max_supported_key_part_length() const;
+  uint8 table_cache_type();
 #ifdef HANDLER_HAS_NEED_INFO_FOR_AUTO_INC
-  bool need_info_for_auto_inc() override;
+  bool need_info_for_auto_inc();
 #endif
 #ifdef HANDLER_HAS_CAN_USE_FOR_AUTO_INC_INIT
-  bool can_use_for_auto_inc_init() override;
+  bool can_use_for_auto_inc_init();
 #endif
   int update_auto_increment();
   void get_auto_increment(
@@ -470,25 +498,25 @@ public:
     ulonglong nb_desired_values,
     ulonglong *first_value,
     ulonglong *nb_reserved_values
-  ) override;
+  );
   int reset_auto_increment(
     ulonglong value
-  ) override;
-  void release_auto_increment() override;
+  );
+  void release_auto_increment();
 #ifdef SPIDER_HANDLER_START_BULK_INSERT_HAS_FLAGS
   void start_bulk_insert(
     ha_rows rows,
     uint flags
-  ) override;
+  );
 #else
   void start_bulk_insert(
     ha_rows rows
   );
 #endif
-  int end_bulk_insert() override;
+  int end_bulk_insert();
   int write_row(
     const uchar *buf
-  ) override;
+  );
 #ifdef HA_CAN_BULK_ACCESS
   int pre_write_row(
     uchar *buf
@@ -500,21 +528,21 @@ public:
     bool hs_request
   );
 #endif
-  bool start_bulk_update() override;
+  bool start_bulk_update();
   int exec_bulk_update(
     ha_rows *dup_key_found
-  ) override;
-  int end_bulk_update() override;
+  );
+  int end_bulk_update();
 #ifdef SPIDER_UPDATE_ROW_HAS_CONST_NEW_DATA
   int bulk_update_row(
     const uchar *old_data,
     const uchar *new_data,
     ha_rows *dup_key_found
-  ) override;
+  );
   int update_row(
     const uchar *old_data,
     const uchar *new_data
-  ) override;
+  );
 #else
   int bulk_update_row(
     const uchar *old_data,
@@ -564,7 +592,7 @@ public:
 #ifdef SPIDER_MDEV_16246
   int direct_update_rows_init(
     List<Item> *update_fields
-  ) override;
+  );
 #else
   int direct_update_rows_init();
 #endif
@@ -625,7 +653,7 @@ public:
   int direct_update_rows(
     ha_rows *update_rows,
     ha_rows *found_row
-  ) override;
+  );
 #endif
 #ifdef HA_CAN_BULK_ACCESS
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
@@ -650,11 +678,11 @@ public:
 #endif
 #endif
 #endif
-  bool start_bulk_delete() override;
-  int end_bulk_delete() override;
+  bool start_bulk_delete();
+  int end_bulk_delete();
   int delete_row(
     const uchar *buf
-  ) override;
+  );
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
   bool check_direct_delete_sql_part(
     st_select_lex *select_lex,
@@ -673,7 +701,7 @@ public:
     bool sorted
   );
 #else
-  int direct_delete_rows_init() override;
+  int direct_delete_rows_init();
 #endif
 #ifdef HA_CAN_BULK_ACCESS
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
@@ -705,7 +733,7 @@ public:
 #else
   int direct_delete_rows(
     ha_rows *delete_rows
-  ) override;
+  );
 #endif
 #ifdef HA_CAN_BULK_ACCESS
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
@@ -726,90 +754,90 @@ public:
 #endif
 #endif
 #endif
-  int delete_all_rows() override;
-  int truncate() override;
-  double scan_time() override;
+  int delete_all_rows();
+  int truncate();
+  double scan_time();
   double read_time(
     uint index,
     uint ranges,
     ha_rows rows
-  ) override;
+  );
 #ifdef HA_CAN_BULK_ACCESS
   void bulk_req_exec();
 #endif
-  const key_map *keys_to_use_for_scanning() override;
-  ha_rows estimate_rows_upper_bound() override;
+  const key_map *keys_to_use_for_scanning();
+  ha_rows estimate_rows_upper_bound();
   void print_error(
     int error,
     myf errflag
-  ) override;
+  );
   bool get_error_message(
     int error,
     String *buf
-  ) override;
+  );
   int create(
     const char *name,
     TABLE *form,
     HA_CREATE_INFO *info
-  ) override;
+  );
   void update_create_info(
     HA_CREATE_INFO* create_info
-  ) override;
+  );
   int rename_table(
     const char *from,
     const char *to
-  ) override;
+  );
   int delete_table(
     const char *name
-  ) override;
-  bool is_crashed() const override;
+  );
+  bool is_crashed() const;
 #ifdef SPIDER_HANDLER_AUTO_REPAIR_HAS_ERROR
-  bool auto_repair(int error) const override;
+  bool auto_repair(int error) const;
 #else
   bool auto_repair() const;
 #endif
   int disable_indexes(
-    key_map map, bool persist
-  ) override;
+    uint mode
+  );
   int enable_indexes(
-    key_map map, bool persist
-  ) override;
+    uint mode
+  );
   int check(
     THD* thd,
     HA_CHECK_OPT* check_opt
-  ) override;
+  );
   int repair(
     THD* thd,
     HA_CHECK_OPT* check_opt
-  ) override;
+  );
   bool check_and_repair(
     THD *thd
-  ) override;
+  );
   int analyze(
     THD* thd,
     HA_CHECK_OPT* check_opt
-  ) override;
+  );
   int optimize(
     THD* thd,
     HA_CHECK_OPT* check_opt
-  ) override;
+  );
   bool is_fatal_error(
     int error_num,
     uint flags
-  ) override;
+  );
   Field *field_exchange(
     Field *field
   );
   const COND *cond_push(
     const COND* cond
-  ) override;
-  void cond_pop() override;
+  );
+  void cond_pop();
   int info_push(
     uint info_type,
     void *info
-  ) override;
+  );
 #ifdef HANDLER_HAS_DIRECT_AGGREGATE
-  void return_record_by_parent() override;
+  void return_record_by_parent();
 #endif
   TABLE *get_table();
   void set_ft_discard_bitmap();
@@ -1142,26 +1170,3 @@ public:
   void bulk_access_end();
 #endif
 };
-
-
-/* This is a hack for ASAN
- * Libraries such as libxml2 and libodbc do not like being unloaded before
- * exit and will show as a leak in ASAN with no stack trace (as the plugin
- * has been unloaded from memory).
- *
- * The below is designed to trick the compiler into adding a "UNIQUE" symbol
- * which can be seen using:
- * readelf -s storage/spider/ha_spider.so | grep UNIQUE
- *
- * Having this symbol means that the plugin remains in memory after dlclose()
- * has been called. Thereby letting the libraries clean up properly.
- */
-#if defined(__SANITIZE_ADDRESS__)
-__attribute__((__used__))
-inline int dummy(void)
-{
-  static int d;
-  d++;
-  return d;
-}
-#endif
